/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.platform;

import org.eclipse.draw2d.graph.DirectedGraph;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.PhenotypeBase;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.copy.Copyable;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.graph.display.JGraphTVisualizer;
import org.fortiss.af3.exploration.alg.service.IExplorationEncoding;
import org.jgrapht.graph.DefaultDirectedGraph;
import org.jgrapht.graph.DefaultEdge;
import org.opt4j.core.Genotype;

/**
 * Abstracts the communication network present in a platform system meta-model to a flat graph. This
 * graph is needed to manage the communication of logical {@link ITaskAdapter}s after
 * they have been deployed onto a concrete platform. More precisely, the graph is used for routing
 * of (physical) messages and their scheduling on the communication resources, like busses.
 * 
 * @author diewald
 */
public class PlatformCommunicationGraphEncoding extends PhenotypeBase
		implements IExplorationEncoding, Genotype, Copyable<PlatformCommunicationGraphEncoding> {
	/** The graph representing the communication structure of a platform. */
	protected DefaultDirectedGraph<IResourceAdapter<?>, DefaultEdge> platformGraph;

	/** Constructor for the initial version of the Graph w/o available Partitions. */
	public PlatformCommunicationGraphEncoding(
			DefaultDirectedGraph<IResourceAdapter<?>, DefaultEdge> platformGraph) {
		this.platformGraph = platformGraph;
	}

	/** Copy constructor. */
	protected PlatformCommunicationGraphEncoding() {
	}

	/** Creates a Dialog that illustrates this {@link DirectedGraph}. */
	public void display() {
		JGraphTVisualizer<IResourceAdapter<?>, DefaultEdge> taskViz =
				new JGraphTVisualizer<IResourceAdapter<?>, DefaultEdge>(platformGraph);
		taskViz.displayGraph("Platform communication graph");
	}

	/**
	 * Returns the actual graph that contains information about the {@link IResourceAdapter}s and
	 * their connections.
	 */
	public DefaultDirectedGraph<IResourceAdapter<?>, DefaultEdge> getActualGraph() {
		return platformGraph;
	}

	/** {@inheritDoc} */
	@Override
	public int size() {
		return platformGraph.vertexSet().size() + platformGraph.edgeSet().size();
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public PlatformCommunicationGraphEncoding newInstance() {
		return new PlatformCommunicationGraphEncoding(platformGraph);
	}

	/** {@inheritDoc} */
	@Override
	public Class<? extends Phenotype> getPTypeToRegister() {
		return PlatformCommunicationGraphEncoding.class;
	}

	/** {@inheritDoc} */
	@Override
	public PlatformCommunicationGraphEncoding copy() {
		return new PlatformCommunicationGraphEncoding(platformGraph);
	}
}
