/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph;

import java.util.Collection;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.taskgraph.TaskGraph;

/**
 * {@link TaskGraphEncoding} which provides {@link TaskGraph}s for explored architectures of safety
 * functions that are encoded by a {@link SafetyFunctionArchEncoding}.
 * 
 * @author diewald
 */
public class SafeTaskGraphEncoding extends AbstractTaskGraphEncoding {
	/** Collection of task graphs extracted from the input models. */
	private Collection<TaskGraph> originalTaskGraphs;

	/** Constructor. */
	public SafeTaskGraphEncoding(Set<TaskGraph> originalTaskGraphs, Double hyperPeriod) {
		super(originalTaskGraphs, hyperPeriod);
		this.originalTaskGraphs = originalTaskGraphs;
	}

	/** Copy Constructor. See {@link TaskGraphEncoding#TaskGraphEncoding(TaskGraphEncoding)}. */
	public SafeTaskGraphEncoding(SafeTaskGraphEncoding encoding) {
		super(encoding);
	}

	/**
	 * Detects whether the {@link TaskGraph} of the input system model references any tasks of the
	 * internal {@link TaskGraph}s and returns the first matching {@link TaskGraph}.
	 */
	public TaskGraph getOriginalTaskGraph(ITaskAdapter<?> task) {
		for(TaskGraph currTG : originalTaskGraphs) {
			if(currTG.containsTask(task)) {
				return currTG;
			}
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public Class<? extends Phenotype> getPTypeToRegister() {
		return SafeTaskGraphEncoding.class;
	}
}
