/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode;

import java.util.Collection;
import java.util.Collections;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype.ComposableGenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;

import com.google.common.reflect.TypeToken;
import com.google.inject.AbstractModule;

/**
 * Base class for {@link ComposableDecoder}s.
 * 
 * @author diewald
 */
public abstract class ComposableDecoderBase<G extends ComposableGenotype, P extends Phenotype>
		extends AbstractModule implements ComposableDecoder<G, P> {
	/** Type information: Concrete input genotype of the decoder. */
	private TypeToken<G> genotype = new TypeToken<G>(getClass()) { // Empty class
	};
	/** Type information: Concrete input phenotype of the decoder. */
	private TypeToken<P> phenotype = new TypeToken<P>(getClass()) { // Empty class
	};

	/** {@inheritDoc} */
	@Override
	public void configure() {
		// Nothing to be done here by default.
	}

	/** {@inheritDoc} */
	@Override
	public Class<?> getPrimaryInputType() {
		if(genotype.getType() instanceof Class) {
			return (Class<?>)genotype.getType();
		}
		return genotype.getRawType();
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getOptionalTypes() {
		return Collections.emptySet();
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public Class<? super P> getOutputType() {
		if(phenotype.getType() instanceof Class) {
			return (Class<? super P>)phenotype.getType();
		} else if(phenotype.getRawType() != null) {
			return phenotype.getRawType();
		}
		return null;
	}
}
