/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.EncodingCache;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.thread.ThreadScoped;

import com.google.inject.Provides;

/**
 * Provides an {@link EncodingCache} for {@link Phenotype}s that collects all sub {@link Phenotype}s
 * of a single Individual in the decode phase. This cache is thread-safe as it provides each
 * requesting thread with a different instance.
 * 
 * @author diewald
 */
public class PhenotypeCache extends EncodingCache<Phenotype> {

	/**
	 * Returns an instance of the {@link PhenotypeCache} per thread. If the cache is requested from
	 * the same thread more than once, the existing instance will be provided, which is guaranteed
	 * by the {@code @ThreadScoped} annotation.
	 */
	@Provides
	@ThreadScoped
	public PhenotypeCache getEncodingCache() {
		return new PhenotypeCache();
	}
}
