/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype;

import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype.ComposableGenotype;
import org.fortiss.af3.exploration.alg.service.IExplorationEncoding;
import org.opt4j.core.genotype.CompositeGenotype;

/**
 * Interface for {@link Phenotype}s generated by decoders which operate on {@link CompositeGenotype}
 * s.
 * 
 * @author diewald
 */
public interface Phenotype
		extends IExplorationEncoding, Iterable<Entry<Class<? extends Phenotype>, Phenotype>> {

	/**
	 * Returns the type which shall be used to register the {@link Phenotype} at its
	 * containing {@link Phenotype}.
	 */
	Class<? extends Phenotype> getPTypeToRegister();

	/**
	 * Returns the {@link Phenotype} that corresponds to the given {@link Phenotype} class.
	 */
	<CP extends Phenotype> CP getSubPhenotype(Class<CP> subPhenotype);

	/**
	 * Defines a correspondence between a {@link ComposableGenotype} class and its {@link Phenotype}
	 * .
	 */
	<CP extends Phenotype> void putSubPhenotype(Class<CP> subGenotype, CP curPhenotype);

	/**
	 * Returns the collection of {@link Phenotype}s that make up this {@link ComposableGenotype}.
	 */
	List<Phenotype> getSubPhenotypes();

	/** Returns the Class -> {@link Phenotype} entries of the enclosed map. */
	Set<Class<? extends Phenotype>> keySet();

	/** Marks a phenotype as being processed by sub-decoders. */
	void setSubDecoding(boolean isSubDecoding);

	/** Returns whether the {@link Phenotype} is currently processed by sub-decoders. */
	boolean isSubDecoding();
}
