/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.EncodingProviderModule;
import org.opt4j.core.Genotype;

import com.google.common.collect.ClassToInstanceMap;

/**
 * Container of the {@link Genotype} and {@link Phenotype} providers.
 * 
 * @author diewald
 */
public class ProvidingPhenotype extends PhenotypeBase {

	/** Holds the reference to the {@link EncodingProviderModule} of {@link Genotype} providers. */
	private EncodingProviderModule<Genotype> genotypePoviderModule;

	/** Holds the reference to the {@link EncodingProviderModule} produced by the decoding phase. */
	private EncodingProviderModule<Phenotype> phenotypePoviderModule;

	/** References the map of solution encodings. */
	// TODO: This field is for legacy support (transformation). We can switch the transformation
	// mechanism to use DI as well.
	private ClassToInstanceMap<Phenotype> phenotypeMap;

	/** References the map of solution encodings. */
	// TODO: This field is for legacy support (transformation). We can switch the transformation
	// mechanism to use DI as well.
	private ClassToInstanceMap<Genotype> genotypeMap;

	/** Constructor. */
	public ProvidingPhenotype(EncodingProviderModule<Genotype> genotypePoviderModule,
			EncodingProviderModule<Phenotype> phenotypePoviderModule,
			ClassToInstanceMap<Phenotype> phenotypeMap, ClassToInstanceMap<Genotype> genotypeMap) {
		this.genotypePoviderModule = genotypePoviderModule;
		this.phenotypePoviderModule = phenotypePoviderModule;
		this.phenotypeMap = phenotypeMap;
		this.genotypeMap = genotypeMap;
	}

	/** Returns the {@link EncodingProviderModule} holding Providers for Genotypes. */
	public EncodingProviderModule<Genotype> getGenotypeProviderModule() {
		return genotypePoviderModule;
	}

	/** Returns the {@link EncodingProviderModule} holding Providers for Phenotypes. */
	public EncodingProviderModule<Phenotype> getPhenotypeProviderModule() {
		return phenotypePoviderModule;
	}

	/** Returns the map of solution encodings. */
	public ClassToInstanceMap<Phenotype> getPhenotypeMap() {
		return phenotypeMap;
	}

	/**
	 * Returns the map of {@link Genotype}s used to construct the {@link Phenotype}s of this
	 * solution candidate.
	 */
	public ClassToInstanceMap<Genotype> getGenotypeMap() {
		return this.genotypeMap;
	}
}
